==> lkrg-0.0.tar.gz: lkrg-0.0/src/modules/exploit_detection/syscalls/p_sys_finit_module/p_sys_finit_module.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  On kernel < 4.0 you can't intercept do_init_module function
 *  and each *init_module syscall must be intercepted individually.
 *  This module intercepts sys_finit_module
 *
 * Notes:
 *  - Enforce Exploit Detection validation
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.XII.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_sys_finit_module_kretprobe_state = 0x0;

static struct kretprobe p_sys_finit_module_kretprobe = {
    .kp.symbol_name = "sys_finit_module",
    .handler = p_sys_finit_module_ret,
    .entry_handler = p_sys_finit_module_entry,
    .data_size = sizeof(struct p_sys_finit_module_data),
    /* Probe up to 20 instances concurrently. */
    .maxactive = 40,
};


int p_sys_finit_module_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_sys_finit_module_entry>\n");

   p_ed_enforce_validation();
   if (!capable(CAP_SYS_MODULE)) {
      struct module *p_tmp_mod = (struct module *)p_regs->di;

      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Process[%s | %d] is trying to load a kernel module[%s] "
             "but does NOT have appropiate permissions! Blocking...\n",
             current->comm,task_pid_nr(current),p_tmp_mod->name);

      p_tmp_mod->init = p_block_always;
      // kill this process!
      p_ed_kill_task_by_task(current);
   }

   p_debug_kprobe_log(
          "Leaving function <p_sys_finit_module_entry>\n");

   return 0x0;
}


int p_sys_finit_module_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_sys_finit_module_ret>\n");

   p_ed_enforce_validation();
   if (!capable(CAP_SYS_MODULE)) {

      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Process[%s | %d] loaded kernel module but does NOT have "
             "appropiate permissions! Killing...\n",
             current->comm,task_pid_nr(current));

      // kill this process!
      p_ed_kill_task_by_task(current);
   }

   p_debug_kprobe_log(
          "Leaving function <p_sys_finit_module_ret>\n");

   return 0x0;
}


int p_install_sys_finit_module_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_sys_finit_module_hook>\n");

   if ( (p_ret = register_kretprobe(&p_sys_finit_module_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() failed! [err=%d]\n",p_ret);
      goto p_install_sys_finit_module_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
                         p_sys_finit_module_kretprobe.kp.symbol_name, p_sys_finit_module_kretprobe.kp.addr);
   p_sys_finit_module_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_sys_finit_module_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_sys_finit_module_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_sys_finit_module_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_sys_finit_module_hook>\n");

   if (!p_sys_finit_module_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                     p_sys_finit_module_kretprobe.kp.symbol_name, p_sys_finit_module_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_sys_finit_module_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p\n",
                     p_sys_finit_module_kretprobe.kp.symbol_name, p_sys_finit_module_kretprobe.kp.addr);
      p_sys_finit_module_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_sys_finit_module_hook>\n");
}

==> lkrg-0.1.tar.gz: lkrg-0.1/src/modules/exploit_detection/syscalls/p_sys_finit_module/p_sys_finit_module.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  On kernel < 4.0 you can't intercept do_init_module function
 *  and each *init_module syscall must be intercepted individually.
 *  This module intercepts sys_finit_module
 *
 * Notes:
 *  - Enforce Exploit Detection validation
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.XII.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_sys_finit_module_kretprobe_state = 0x0;

static struct kretprobe p_sys_finit_module_kretprobe = {
    .kp.symbol_name = "sys_finit_module",
    .handler = p_sys_finit_module_ret,
    .entry_handler = p_sys_finit_module_entry,
    .data_size = sizeof(struct p_sys_finit_module_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_finit_module_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_sys_finit_module_entry>\n");

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - validate 'off' flag
      p_ed_is_off_off_wrap(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_ed_enforce_validation();
   if (!capable(CAP_SYS_MODULE)) {
      struct module *p_tmp_mod = (struct module *)p_regs->di;

      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Process[%s | %d] is trying to load a kernel module[%s] "
             "but does NOT have appropriate permissions! Blocking...\n",
             current->comm,task_pid_nr(current),p_tmp_mod->name);

      p_tmp_mod->init = p_block_always;
      // kill this process!
      p_ed_kill_task_by_task(current);
   }

   p_debug_kprobe_log(
          "Leaving function <p_sys_finit_module_entry>\n");

   return 0x0;
}


int p_sys_finit_module_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_sys_finit_module_ret>\n");

   p_ed_enforce_validation();
   if (!capable(CAP_SYS_MODULE)) {

      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Process[%s | %d] loaded kernel module but does NOT have "
             "appropriate permissions! Killing...\n",
             current->comm,task_pid_nr(current));

      // kill this process!
      p_ed_kill_task_by_task(current);
   }

   p_debug_kprobe_log(
          "Leaving function <p_sys_finit_module_ret>\n");

   return 0x0;
}


int p_install_sys_finit_module_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_sys_finit_module_hook>\n");

   if ( (p_ret = register_kretprobe(&p_sys_finit_module_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() failed! [err=%d]\n",p_ret);
      goto p_install_sys_finit_module_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
                         p_sys_finit_module_kretprobe.kp.symbol_name, p_sys_finit_module_kretprobe.kp.addr);
   p_sys_finit_module_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_sys_finit_module_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_sys_finit_module_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_sys_finit_module_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_sys_finit_module_hook>\n");

   if (!p_sys_finit_module_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                     p_sys_finit_module_kretprobe.kp.symbol_name, p_sys_finit_module_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_sys_finit_module_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p\n",
                     p_sys_finit_module_kretprobe.kp.symbol_name, p_sys_finit_module_kretprobe.kp.addr);
      p_sys_finit_module_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_sys_finit_module_hook>\n");
}
